@@ -112,7 +112,7 @@ class NodeNotReachableException(Exception):
     pass
 
 
-def _filter_order(queryset, field_names, values):
+def _filter_order(queryset, field_names, values, extra_filters=None):
     """
     Filters the provided queryset for 'field_name__in values' for each given field_name in [field_names]
     orders results in the same order as provided values
@@ -120,6 +120,10 @@ def _filter_order(queryset, field_names, values):
         For instance
             _filter_order(self.__class__.objects, "pk", ids)
         returns a queryset of the current class, with instances where the 'pk' field matches an id in ids
+        
+    Extra filters can be added via a dictionary of lookups.
+    For instance, assuming the target model has a "name" field:
+        _filter_order(self.__class__.objects, "pk", ids, extra_filters={"name__icontains", "Jack"})
     """
     if not isinstance(field_names, list):
         field_names = [field_names]
@@ -129,6 +133,8 @@ def _filter_order(queryset, field_names, values):
         case.append(When(**when_condition))
     order_by = Case(*case)
     filter_condition = {field_name + "__in": values for field_name in field_names}
+    if extra_filters is not None:
+        filter_condition.update(extra_filters)
     return queryset.filter(**filter_condition).order_by(order_by)
 
 
@@ -226,9 +232,9 @@ def node_factory(edge_model, children_null=True, base_model=models.Model):
             """
             return self.filter_order_ids(self.clan_ids())
 
-        def descendants_edges_set(self, cached_results=None):
+        def descendants_edges(self, cached_results=None):
             """
-            Returns a set of descendants edges
+            Returns a queryset of descendants edges
             # ToDo: Modify to use CTE
             """
             if cached_results is None:
@@ -236,16 +242,17 @@ def node_factory(edge_model, children_null=True, base_model=models.Model):
             if self.id in cached_results.keys():
                 return cached_results[self.id]
             else:
-                res = set()
+                edge_set = set()
                 for f in self.children.all():
-                    res.add(edge_model.objects.get(parent=self.id, child=f.id))
-                    res.update(f.descendants_edges_set(cached_results=cached_results))
-                cached_results[self.id] = res
-                return res
+                    edge_set.add(edge_model.objects.get(parent=self.id, child=f.id))
+                    edge_set.update(f.descendants_edges(cached_results=cached_results))
+                cached_results[self.id] = edge_set
+            out = [x.id for x in edge_set]
+            return _filter_order(edge_model.objects, "pk", out)
 
-        def ancestors_edges_set(self, cached_results=None):
+        def ancestors_edges(self, cached_results=None):
             """
-            Returns a set of ancestors edges
+            Returns a queryset of ancestors edges
             # ToDo: Modify to use CTE
             """
             if cached_results is None:
@@ -253,29 +260,30 @@ def node_factory(edge_model, children_null=True, base_model=models.Model):
             if self in cached_results.keys():
                 return cached_results[self.id]
             else:
-                res = set()
+                edge_set = set()
                 for f in self.parents.all():
-                    res.add(edge_model.objects.get(child=self.id, parent=f.id))
-                    res.update(f.ancestors_edges_set(cached_results=cached_results))
-                cached_results[self.id] = res
-                return res
+                    edge_set.add(edge_model.objects.get(child=self.id, parent=f.id))
+                    edge_set.update(f.ancestors_edges(cached_results=cached_results))
+                cached_results[self.id] = edge_set
+            out = [x.id for x in edge_set]
+            return _filter_order(edge_model.objects, "pk", out)
 
-        def clan_edges_set(self):
+        def clan_edges(self):
             """
-            Returns a set of all edges associated with a given node
+            Returns a queryset of all edges associated with a given node
             """
             edges = set()
-            edges.update(self.descendants_edges_set())
-            edges.update(self.ancestors_edges_set())
+            edges.update(self.descendants_edges())
+            edges.update(self.ancestors_edges())
             return edges
 
         def path_ids_list(
             self, target_node, directional=True, max_depth=20, max_paths=1
         ):
             """
-            Returns a list of paths from self to target node, in either direction.
-            The resulting lists are always sorted from top of graph, downward,
-            regardless of the relative position of starting and ending nodes.
+            Returns a list of paths from self to target node, optionally in either 
+            direction. The resulting lists are always sorted from root-side, toward
+            leaf-side, regardless of the relative position of starting and ending nodes.
 
             By default, returns only one shortest path, but additional paths
             can be included by setting the max_paths argument.
